"""Top-level package for datacatalog_util."""

from .datacatalog_util_cli import main  # noqa: F401

__author__ = """Marcelo Miranda"""
__email__ = 'mesmacosta@gmail.com'
__version__ = '0.11.1'

__all__ = ('main')
